%option full
%option noyywrap
%option case-insensitive
%option nostdinit
%option yylineno

%x read_states
%x read_initial_state
%x read_accepting_states
%x read_rejecting_states
%x read_alphabet
%x read_blank_symbol
%x read_delta
%x read_transition
%x read_descriptions
%x read_description

%{
	int atoi(char* x) {
		if(x[0] == '-')
			return -atoi(x+1);
		int result = 0;
		for(int i = 0; x[i] != '\0'; i++)
			result = result*10+(x[i]-'0');
		return result;
	}
%}

%%

"//"[^\n]*                          {}
[\ \n\t\r]+							{}
"states"							{return STATES;}
"initial state"						{return INITIALSTATE;}
"accepting states"					{return ACCEPTINGSTATES;}
"rejecting states"					{return REJECTINGSTATES;}
"alphabet"							{return ALPHABET;}
"descriptions"						{return DESCRIPTIONS;}
"blank symbol"						{return BLANKSYMBOL;}
"delta"|"transitions"				{return DELTA;}

"("									{return OPEN;}
")"									{return CLOSE;}
";"									{return SEMICOLON;}
":"									{return COLON;}
","									{return COMMA;}
"="									{return EQ;}
"-"									{return NIL;}
[a-zA-Z][0-9a-zA-Z]*				{yylval.s = new(string)(yytext);   return ID;}
"-"?[0-9]+							{yylval.i = atoi(yytext);          return NUM;}
"'"."'"|\\.							{yylval.i = yytext[1];             return CHAR;}
\"[^\"]*\"							{yytext[yyleng-1]='\0'; yylval.s = new(string)(yytext+1); return STRING;}
.									{cerr << "LEXICAL ERROR (Line "<<yylineno<<"): unexpected character \""<<yytext<<"\"\n"; yynerrs++;}

%%
