%{
// TuringMachines are modelled as a class so maybe some day the simulator
// might get the ability to have sub-programs.
// negative states = sub-programs ?
// alphabet: suppress multiple declarations of chars


#define INTERVAL // maybe disable on windows (usleep may be unavailiable)

#include<iostream>
#include<map>
#include<set>
#include<string>
#include<vector>
#include<stdio.h>

#ifdef INTERVAL
	#include <unistd.h>
#endif
using namespace std;
#define YYERROR_VERBOSE

#ifdef INTERVAL
	int interval = 41666;
#endif
bool print_steps=true;
bool verbose=false;
string pre_tape="  ";
string post_tape="\r";
string pre_head=" [";
string post_head="] ";
string pre_run="\n";
string post_run="\n\n";

int yyparse();
int yylex();
void yyerror(const char* s);
extern FILE* yyin;
extern int yylineno;

class TuringMachine {
	public:
		// static map<string, TuringMachine*> Machines;
		static TuringMachine *current;
		int alphacounter;

		// make these static?
		vector<char> tape;
		int head_position;
		vector<char> alphabet;
		char blank_symbol;

		int state;
		int initial_state;
		map<string, int> state_of_name;
		map<int, string> name_of_state;
		set<int> accepting_state;
		set<int> rejecting_state;

		int phase;
		map<int, int> phase_of_state;
		map<int, string> phase_description;

		map<int, map<char, vector<int> > > delta;
		// delta[state][read char] = [new state, new char, head movement]

		void print() {
			if(print_steps) {
				if(verbose && phase_of_state.find(state) != phase_of_state.end()) {
					if(phase != phase_of_state[state]) {
						phase = phase_of_state[state];
						if(phase != -1)
							cout << phase_description[phase] << "\n";
					}
				}
				cout << pre_tape << name_of_state[state] << " : ";
				for(int i = 0; i < head_position; i++)
					cout << tape[i];
				cout << pre_head << tape[head_position] << post_head;
				#ifdef INTERVAL
					if(interval > 0)
						usleep(interval);
				#endif

				for(int i = head_position+1; i < tape.size(); i++)
					cout << tape[i];
				cout << post_tape;cout.flush();
			}
		}

		bool run() {
			if(print_steps)
				cout << pre_run;
			state = initial_state;
			print();
			vector<int> delta_result;
			map<int, map<char, vector<int> > >::iterator i;
			map<char, vector<int> >::iterator j;

			while(accepting_state.find(state) == accepting_state.end() &&
			      rejecting_state.find(state) == rejecting_state.end()) {

				i = delta.find(state);
				if(i == delta.end()) {
					cerr << "\n\n  ERROR: No transitions defined for state \"" << name_of_state[state] << "\"\n";
					break;
				}
				j = i->second.find(tape[head_position]);
				if(j == i->second.end()) {
					cerr << "\n\n  ERROR: State \"" << name_of_state[state] << "\" has no transition for symbol \'" << tape[head_position] << "\'\n";
					break;
				}

				delta_result = j->second;
				state = delta_result[0];
				tape[head_position] = (char)(delta_result[1]);
				head_position += delta_result[2];
				if(head_position < 0) {
					vector<char> temp(-head_position, blank_symbol);
					tape.insert(tape.begin(), temp.begin(), temp.end());
					head_position = 0;
				}
				if(head_position >= tape.size()) {
					vector<char> temp(head_position-tape.size()+1,blank_symbol);
					tape.insert(tape.end(), temp.begin(), temp.end());
				}
				print();
			}
			if(print_steps)
				cout << post_run;
			return(accepting_state.find(state) != accepting_state.end());
		}
		TuringMachine(string FileName) {
			phase = 1;
			state = 1;
			alphacounter = 0;
			
			TuringMachine* CurrentBak = current;
			FILE* yyinBak = yyin;
			current = this;
			yyin = fopen(FileName.c_str(),"r");
			yyparse();
			fclose(yyin);
			yyin = yyinBak;
			current = CurrentBak;
			rejecting_state.insert(0);
		}
		void EditState(string name, int _phase, bool accepting, bool rejecting, bool start) {
			int s;
			if(state_of_name.find(name) != state_of_name.end()) {
				s = state_of_name[name];
			} else {
				s = state++;
				state_of_name[name] = s;
				name_of_state[s] = name;
			}
			if(_phase != 0)
				phase_of_state[s] = _phase;
			if(accepting)
				accepting_state.insert(s);
			if(rejecting)
				rejecting_state.insert(s);
			if(start)
				initial_state = s;
		}
};
TuringMachine* TuringMachine::current = NULL;

%} // Yacc Deklarationen

%union {int i; char *c; string *s; vector<int> *v; map<char, vector<int> > *m;}

%start S
%token STATES INITIALSTATE ACCEPTINGSTATES REJECTINGSTATES ALPHABET
       DESCRIPTIONS BLANKSYMBOL DELTA OPEN CLOSE SEMICOLON COMMA
       COLON ID CHAR STRING NUM EQ NIL

%% // Produktionen

S: command
 | S command
 ;
command: STATES s_ids SEMICOLON
       | INITIALSTATE ID SEMICOLON	{TuringMachine::current->EditState(*$<s>1,0,false,false,true); delete($<s>1);}
       | ACCEPTINGSTATES a_ids SEMICOLON
       | REJECTINGSTATES r_ids SEMICOLON
       | BLANKSYMBOL CHAR SEMICOLON {TuringMachine::current->blank_symbol = $<i>2; }
       | ALPHABET sigma SEMICOLON
       | DELTA deltas SEMICOLON
       | DESCRIPTIONS descr SEMICOLON
       ;

s_ids: s_ids COMMA ID				{TuringMachine::current->EditState(*$<s>3,0,false,false,false); delete($<s>3);}
    | ID 							{TuringMachine::current->EditState(*$<s>1,0,false,false,false); delete($<s>1);}
    ;
a_ids: a_ids COMMA ID				{TuringMachine::current->EditState(*$<s>3,0,true,false,false); delete($<s>3);}
    | ID 							{TuringMachine::current->EditState(*$<s>1,0,true,false,false); delete($<s>1);}
    ;
r_ids: r_ids COMMA ID				{TuringMachine::current->EditState(*$<s>3,0,false,true,false); delete($<s>3);}
    | ID 							{TuringMachine::current->EditState(*$<s>1,0,false,true,false); delete($<s>1);}
    ;
sigma: CHAR							{TuringMachine::current->alphabet.push_back($<i>1);}
     | sigma COMMA CHAR				{TuringMachine::current->alphabet.push_back($<i>3);}
	 | NUM							{if(0 <= $<i>1 && 9 >= $<i>1)
										TuringMachine::current->alphabet.push_back($<i>1+'0');
									else {
										cerr << "SYNTACTIC ERROR (Line "<<yylineno<<"): syntax error, unexpected NUM, expecting 0..9\n";
										yynerrs++;
									}}
     | sigma COMMA NUM				{if(0 <= $<i>3 && 9 >= $<i>3)
										TuringMachine::current->alphabet.push_back($<i>3+'0');
									else {
										cerr << "SYNTACTIC ERROR (Line "<<yylineno<<"): syntax error, unexpected NUM, expecting 0..9\n";
										yynerrs++;
									}}
     ;

deltas: 
     | deltas ID COLON {TuringMachine::current->alphacounter=0;} transitions SEMICOLON {TuringMachine::current->delta[TuringMachine::current->state_of_name[*$<s>2]] = *$<m>5; delete($<m>5); delete($<s>2);}
     ;
transitions: transitions COMMA transition {$<m>$=$<m>1; (*$<m>$)[TuringMachine::current->alphabet[TuringMachine::current->alphacounter++]] = *$<v>3; delete($<v>3);}
           | transition {$<m>$ = new map<char,vector<int> >; (*$<m>$)[TuringMachine::current->alphabet[TuringMachine::current->alphacounter++]] = *$<v>1; delete($<v>1);}
           ;
transition: OPEN ID COMMA CHAR COMMA NUM CLOSE {$<v>$ = new vector<int>(3); (*$<v>$)[0] = TuringMachine::current->state_of_name[*$<s>2]; delete($<s>2); (*$<v>$)[1] = $<i>4; (*$<v>$)[2] = $<i>6;}
          | OPEN ID COMMA CHAR COMMA ID CLOSE {$<v>$ = new vector<int>(3); (*$<v>$)[0] = TuringMachine::current->state_of_name[*$<s>2]; delete($<s>2); (*$<v>$)[1] = $<i>4; (*$<v>$)[2] = -1;}
          | NIL                                {$<v>$ = new vector<int>(3); (*$<v>$)[0] = 0; (*$<v>$)[1] = TuringMachine::current->blank_symbol; (*$<v>$)[2] = 0;}
          ;

descr: 
     | descr d_ids COLON STRING SEMICOLON {TuringMachine::current->phase_description[TuringMachine::current->phase++] = *$<s>4; delete($<s>4);}
     ;
d_ids: d_ids COMMA ID 				{TuringMachine::current->EditState(*$<s>3,TuringMachine::current->phase,false,false,false); delete($<s>3);}
     | ID 							{TuringMachine::current->EditState(*$<s>1,TuringMachine::current->phase,false,false,false); delete($<s>1);}
     ;



%% // C++ Code, der ans Ende der erzeugten Datei kommt

#include "lexer.cpp"


void yyerror(const char* s) {
	cerr << "SYNTACTIC ERROR (Line " << yylineno << "): " << s << endl;
}


int main(int argc, char** argv) {
	bool green=false, highlight=false;
	int i=1;
	for(;i<argc; i++) {
		string argvi = argv[i];
		if(argvi == "--green")
			green = true;
		else if(argvi == "--verbose")
			verbose = true;
		else if(argvi == "--silent")
			print_steps = false;
		else if(argvi == "--highlight")
			highlight = true;
		else if(argvi == "--matrix")
			{green=true; verbose=true; highlight=true;}
		#ifdef INTERVAL
			else if(argvi == "--interval" && i < argc-2)
				interval = atoi(argv[++i]);
		#endif
		else break;
	}
	if(i > argc-2) {
		cerr << "\n usage: " << argv[0] << " [options] <tm-file> <initial tape content>\n";
		cerr << "    or  " << argv[0] << " [options] <tm-file> <tape content> <content behind head>\n";
		cerr << "        where [options] may include\n";
		cerr << "        --green           green output\n";
		cerr << "        --verbose         to print every step in its own line AND display descriptions\n";
		cerr << "        --silent          only print result\n";
		cerr << "        --highlight       use colors instead of brackets to show the head position\n";
		#ifdef INTERVAL
			cerr << "        --interval <int>  time (in microseconds) between steps (default 41666)\n";
		#endif
		cerr << "        --matrix          all 3 together (cool)\n";
		cerr << "        (green and highlight might not work on windows)\n\n";
		return 1;
	}

	if(green) {
		pre_run = "\n\33[32;40m";
		post_run = "\33[37;40m\n\n";
		if(highlight) {
			pre_head = "\33[30;42m";
			post_head = "\33[32;40m";
		}
		if(verbose) {
			post_tape="\n";
			post_run="\33[37;40m\n";
		}
	} else {
		if(highlight) {
			pre_head = "\33[30;47m";
			post_head = "\33[37;40m";
		}
		if(verbose) {
			post_tape="\n";
			post_run="\n";
		}
	}


	TuringMachine M(argv[i]);
	if(yynerrs > 0) {
		cerr << " syntactic errors: " << yynerrs << "\n";
		return 1;
	} else {
		for(int j = 0; argv[i+1][j] != '\0'; j++)
			M.tape.push_back(argv[i+1][j]);
		if(i == argc-2) {
			M.head_position = 0;
		} else {
			M.head_position = M.tape.size();
			for(int j = 0; argv[i+2][j] != '\0'; j++)
				M.tape.push_back(argv[i+2][j]);
		}
		while(M.head_position >= M.tape.size()) // argv[2] or argv[3] was empty
			M.tape.push_back(M.blank_symbol);
		M.run();
		if(!print_steps) {
			for(int i = M.head_position; i < M.tape.size() && M.tape[i] != M.blank_symbol; i++)
				cout << M.tape[i];
			cout << "\n";
		}
		return 0;
	}
}
